### R code from vignette source 'LSqueens.Rnw'

###################################################
### code chunk number 1: LSqueens.Rnw:25-26
###################################################
options(continue = "  ", digits = 5, max.print = 1000)


###################################################
### code chunk number 2: LSqueens.Rnw:62-64
###################################################
require("NMOF")
set.seed(1345767)


###################################################
### code chunk number 3: LSqueens.Rnw:74-77
###################################################
N <- 8              ## board size
p <- sample.int(N)  ## a random solution
data.frame(row = 1:N, column = p)


###################################################
### code chunk number 4: LSqueens.Rnw:81-83
###################################################
p <- rep(1, N) 
data.frame(row = 1:N, column = p)


###################################################
### code chunk number 5: LSqueens.Rnw:89-102
###################################################
print_board <- function(p) {
    n <- length(p)
    row <- rep("-", n)
    for (i in seq_len(n)) {
        row_i <- row
        row_i[p[i]] <- "Q"
       
        cat(paste(row_i, collapse = " "))
        cat("\n")
    }
}

print_board(p)


###################################################
### code chunk number 6: LSqueens.Rnw:112-127
###################################################
mat <- array(NA, dim = c(N,N))  ## diagonals
for (r in 1:N) {
    for (c in 1:N) {
        mat[r,c] <- c - r
    }
}
mat

mat <- array(NA, dim = c(N,N))  ## reverse diagonal
for (r in 1:N) {
    for (c in 1:N) {
        mat[r,c] <- c + r - (N + 1)
    }
}
mat


###################################################
### code chunk number 7: LSqueens.Rnw:141-153
###################################################
n_attacks <- function(p) {
    ## more than one Q on a column?
    sum(duplicated(p)) +
        
    ## more than one Q on a diagonal?
    sum(duplicated(p - seq_along(p))) +   

    ## more than one Q on a reverse diagonal?
    sum(duplicated(p + seq_along(p)))
}

n_attacks(p)


###################################################
### code chunk number 8: LSqueens.Rnw:164-175
###################################################
neighbour <- function(p) {
    step <- 3
    i <- sample.int(N, 1)
    p[i] <- p[i] + sample(c(1:step, -(1:step)), 1)
    
    if (p[i] > N)
        p[i] <- 1
    else if (p[i] < 1)
        p[i] <- N
    p
}


###################################################
### code chunk number 9: LSqueens.Rnw:178-181
###################################################
print_board(p)
print_board(p <- neighbour(p))
print_board(p <- neighbour(p))


###################################################
### code chunk number 10: LSqueens.Rnw:189-210
###################################################
## p0 <- sample.int(N)
p0 <- rep(1, N)
print_board(p0)

sol <- LSopt(n_attacks, list(x0 = p0,
                             neighbour = neighbour,
                             printBar = FALSE,
                             nS = 2000))
print_board(sol$xbest)

sol <- TAopt(n_attacks, list(x0 = p0,
                             neighbour = neighbour,
                             printBar = FALSE,
                             nS = 1000))
print_board(sol$xbest)

sol <- SAopt(n_attacks, list(x0 = p0,
                             neighbour = neighbour,
                             printBar = FALSE,
                             nS = 1000))
print_board(sol$xbest)


